.learners-panel {
  width: 750px;
  display: inline-block;
}

.learners-panel .learners-table {
  margin-bottom: 10px;
  width: 100%;
}

.forms-panel {
  vertical-align: top;
  width: 350px;
  display: inline-block;
  margin-left: 20px;
}

.forms-panel .helptext {
  display: block;
  font-size: 0.8em;
  color: #bbb;
}

#id_course_mode {
  width: 150px;
}
